
from rest_framework import viewsets
from django_resaas.models.documento import TipoDocumento
from django_resaas.data.tipo_documento.serializers.tipo_documento import TipoDocumentoSerializer


class  TipoDocumentoAPIView(viewsets.ModelViewSet):
    serializer_class = TipoDocumentoSerializer
    queryset = TipoDocumento.objects.all()
    def get_queryset(self):
        return self.queryset.filter().order_by('-id')

    def update(self, request, *args, **kwargs):
        partial = request.method == 'PATCH'

        instance = self.get_object()

        serializer = self.get_serializer(
            instance,
            data=request.data,
            partial=partial
        )

        serializer.is_valid(raise_exception=True)
        serializer.save()

        return Response(serializer.data)


# seu_app/management/commands/check_metano.py

import pathlib
import sys
from django.core.management.base import BaseCommand, CommandError

FORBIDDEN = [
    "models.Model",
    "ModelSerializer",
    "ModelViewSet",
    "from .",
]


class Command(BaseCommand):
    help = "Verifica se o projeto segue o padrão MetanoStack"

    def add_arguments(self, parser):
        parser.add_argument(
            "--path",
            type=str,
            default=".",
            help="Caminho do projeto (default: .)"
        )
        parser.add_argument(
            "--strict",
            action="store_true",
            help="Falha o comando se encontrar erros"
        )

    def handle(self, *args, **options):
        path = options["path"]
        strict = options["strict"]

        errors = self.check_project(path)

        if errors:
            for e in errors:
                self.stdout.write(self.style.ERROR(e))

            if strict:
                raise CommandError("MetanoStack compliance FAILED")

            return

        self.stdout.write(self.style.SUCCESS("MetanoStack compliance OK"))

    def check_project(self, path="."):
        errors = []
        for py in pathlib.Path(path).rglob("*.py"):
            try:
                text = py.read_text()
            except Exception:
                continue

            for rule in FORBIDDEN:
                if rule in text:
                    errors.append(f"{py}: Forbidden pattern -> {rule}")
        return errors



# =========================
# Django
# =========================
from django.contrib.contenttypes.models import ContentType


# =========================
# Django REST Framework
# =========================
from rest_framework import filters
from rest_framework import status
from rest_framework import viewsets
from rest_framework.response import Response


# =========================
# Local application (absolute imports)
# =========================
from django_resaas.data.modelo.serializers.modelo import ModeloSerializer



class ModeloAPIView(viewsets.ModelViewSet):
    search_fields = ['id']
    filter_backends = (filters.SearchFilter,)
    serializer_class = ModeloSerializer
    queryset = ContentType.objects.all()
    lookup_field = "id"

    def get_queryset(self):
        return self.queryset.order_by('app_label', 'model')

    def list(self, request, *args, **kwargs):
        self._paginator = None
        serializer = self.get_serializer(self.get_queryset(), many=True)
        return Response(serializer.data, status=status.HTTP_200_OK)



from django.db import models
from django.contrib.auth.models import Group
from django_resaas.core.base.models import TimeModel

class SucursalGroup(TimeModel):
    sucursal = models.ForeignKey('django_resaas.Sucursal', on_delete=models.CASCADE)
    group = models.ForeignKey(Group, on_delete=models.CASCADE)

    class Meta:
        unique_together = ("sucursal", "group")
        permissions = ()

    def __str__(self):
        return f'{self.sucursal.nome} | {self.group.name}'




from rest_framework import serializers


from django_resaas.models.sucursal_user_group import SucursalUserGroup


class SucursalUserGroupSerializer(serializers.ModelSerializer):
    class Meta:
        model = SucursalUserGroup
        fields = "__all__"



import base64
from datetime import datetime

import pyotp

from django.conf import settings
from django.core.exceptions import ObjectDoesNotExist

from rest_framework import status
from rest_framework.response import Response
from rest_framework.views import APIView

from django_resaas.core.utils.translate import Translate
from django_resaas.core.utils.username import UserName

from django_resaas.models.user import User
from django_resaas.models.pessoa import Pessoa

from django_resaas.data.user.serializers.register import RegisterSerializer





class RegisterMobileAPIView(APIView):

    @staticmethod
    def post(request):
        phone = str('+' + request.data['mobile']).replace('+', '')

        try:
            user = User.objects.get(mobile=phone)
        except ObjectDoesNotExist:
            if 'reset_senha' in request.query_params:
                return Response(
                    {
                        'alert_error': Translate.tdc(
                            request,
                            'O número não existe'
                        ) + f' <br><b>{phone}</b>'
                    },
                    status=status.HTTP_400_BAD_REQUEST
                )

            nome = (
                request.META['HTTP_ORIGIN']
                .split('.')[0]
                .upper()
                .split('/')[-1]
            )

            request.data['username'] = UserName.Create(
                request.data['username'].replace(' ', '_')
            )
            request.data['mobile'] = phone
            request.data['email'] = None

            serializer = RegisterSerializer(data=request.data)
            serializer.is_valid(raise_exception=True)
            serializer.save()

            user = User.objects.get(mobile=phone)

            pessoa = Pessoa(user=user)
            pessoa.save()

        user.counter += 1
        user.save()

        keygen = generateKeyOTP()
        key = base64.b32encode(keygen.returnValue(phone).encode())
        otp = pyotp.HOTP(key)

        try:
            SMS.send(
                from__='+17244014353',
                to__='+258' + phone,
                text__=Translate.tdc(
                    request,
                    'Não partilhe este código'
                ) + f':\nOTP {otp.at(user.counter)}'
            )
        except Exception as exc:
            return Response(
                {
                    'alert_error': Translate.tdc(
                        request,
                        'Erro ao enviar o OTP'
                    )
                },
                status=status.HTTP_400_BAD_REQUEST
            )

        return Response(
            {
                'alert_success': Translate.tdc(
                    request,
                    'Enviámos um OTP para o seu número'
                ) + f' <br><b>{phone}</b>',
                'data': {'otp': otp.at(user.counter)},
            },
            status=status.HTTP_201_CREATED
        )

    @staticmethod
    def put(request):
        phone = str('+' + request.data['mobile']).replace('+', '')

        try:
            user = User.objects.get(mobile=phone)
        except ObjectDoesNotExist:
            return Response(
                {
                    'alert_error': Translate.tdc(
                        request,
                        'Utilizador não encontrado'
                    )
                },
                status=status.HTTP_404_NOT_FOUND
            )

        keygen = generateKeyOTP()
        key = base64.b32encode(keygen.returnValue(phone).encode())
        otp = pyotp.HOTP(key)

        if otp.verify(request.data['otp'], user.counter):
            user.is_verified_mobile = True
            user.save()

            return Response(
                {
                    'alert_success': Translate.tdc(
                        request,
                        'Autorizado com sucesso'
                    ),
                    'data': {
                        'id': user.id,
                        'otp': request.data['otp'],
                        'mobile': request.data['mobile'],
                    },
                },
                status=status.HTTP_202_ACCEPTED
            )

        return Response(
            {
                'alert_error': Translate.tdc(
                    request,
                    'OTP inválido ou expirado'
                )
            },
            status=status.HTTP_400_BAD_REQUEST
        )



import uuid
from django.db import models
from django_resaas.models.user import User
from django_resaas.core.base.models import TimeModel

class UserLogin(TimeModel):
    user = models.ForeignKey(User, on_delete=models.CASCADE)
    dispositivo = models.TextField(null=True)
    mobile = models.CharField(max_length=100, null=True)
    info = models.TextField(null=True)
    local_lat = models.CharField(max_length=100, null=True)
    local_lon = models.CharField(max_length=100, null=True)
    local_nome = models.CharField(max_length=100, null=True)
   
    is_blocked = models.BooleanField(default=False)

    class Meta:
        permissions = ()

    def __str__(self):
        return self.dispositivo or ''




from rest_framework import serializers


from django_resaas.models.sucursal import Sucursal


class SucursalSerializer(serializers.ModelSerializer):
    class Meta:
        model = Sucursal
        fields = [
            'id',
            'nome',
            'entidade',
        ]




from django.db import models
from django.contrib.auth.models import Group
from django_resaas.core.base.models import TimeModel

class Profissao(TimeModel):
    
    nome = models.CharField(max_length=200, unique=True)

    def __str__(self):
        return self.nome